/*
Copyright 2019 Benji Dial

Permission to use, copy, modify, and/or distribute this
software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this
permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
OF THIS SOFTWARE.
*/

#include <stdint.h>

#ifndef FILES_H
#define FILES_H

struct file_info {
  uint16_t first_sector;
  uint16_t current_sector;
  uint8_t *io_buffer;
  uint32_t position;
};

enum fs_format {
  ISO_9660 = 0
};

#define N_FS_FORMATS 1

enum pt_format {
  NOT_PRESENT = 0,
  UNKNOWN = 1,
  DIRECT = 2,

  //GPT = 3,
  //MBR = 4,
};

#define N_PT_FORMATS 0

struct drive_parts {
  uint8_t format;
  uint8_t n_partitions;
  uint8_t partition_types[10];
  uint32_t partition_offsets[10];
  uint32_t partition_sizes[10];
  void *partition_cache[10];
};

struct drive_parts part_info[26];
struct file_info file_table[65536];
void set_working_drive(uint8_t drive, uint8_t part);
uint16_t open_file(uint8_t *name);
void close_file(uint16_t handle);
uint32_t read_file(uint16_t handle, uint32_t length, void *buffer);
void write_file(uint16_t handle, uint32_t length, void *buffer);
void seek_file(uint16_t handle, int32_t by);
uint32_t get_size(uint16_t handle);
uint32_t read_line_file(uint16_t handle, uint32_t max, void *buffer);
void detect_disk_parts(uint8_t drive);

#endif